@@ -14,7 +14,10 @@
 #include "gl_util.h"
 #include <sys/mman.h>
 
-#define TIFFTAG_FORWARDMATRIX1 50964
+#define DNGTAG_FORWARDMATRIX1 50964
+#define DNGTAG_FORWARDMATRIX2 50965
+#define DNGTAG_CFAREPEATPATTERNDIM 33421
+#define DNGTAG_CFAPATTERN 33422
 
 static const float colormatrix_srgb[] = { 3.2409, -1.5373, -0.4986, -0.9692, 1.8759,
                                           0.0415, 0.0556,  -0.2039, 1.0569 };
@@ -46,6 +49,7 @@ static int output_buffer_height = -1;
 // static bool gain_is_manual;
 static int gain;
 static int gain_max;
+static float balance[3] = { 1.0f, 1.0f, 1.0f };
 
 static bool exposure_is_manual;
 static int exposure;
@@ -60,7 +64,7 @@ static void
 register_custom_tiff_tags(TIFF *tif)
 {
         static const TIFFFieldInfo custom_fields[] = {
-                { TIFFTAG_FORWARDMATRIX1,
+                { DNGTAG_FORWARDMATRIX1,
                   -1,
                   -1,
                   TIFF_SRATIONAL,
@@ -68,6 +72,62 @@ register_custom_tiff_tags(TIFF *tif)
                   1,
                   1,
                   "ForwardMatrix1" },
+                { DNGTAG_FORWARDMATRIX2,
+                  -1,
+                  -1,
+                  TIFF_SRATIONAL,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "ForwardMatrix2" },
+                { DCPTAG_PROFILE_TONE_CURVE,
+                  -1,
+                  -1,
+                  TIFF_FLOAT,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "ProfileToneCurve" },
+                { DCPTAG_PROFILE_HUE_SAT_MAP_DIMS,
+                  -1,
+                  -1,
+                  TIFF_FLOAT,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "ProfileHueSatMapDims" },
+                { DCPTAG_PROFILE_HUE_SAT_MAP_DATA_1,
+                  -1,
+                  -1,
+                  TIFF_FLOAT,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "ProfileHueSatMapData1" },
+                { DCPTAG_PROFILE_HUE_SAT_MAP_DATA_2,
+                  -1,
+                  -1,
+                  TIFF_FLOAT,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "ProfileHueSatMapData2" },
+                { DNGTAG_CFAPATTERN,
+                  -1,
+                  -1,
+                  TIFF_BYTE,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "CFAPattern" },
+                { DNGTAG_CFAREPEATPATTERNDIM,
+                  -1,
+                  -1,
+                  TIFF_SHORT,
+                  FIELD_CUSTOM,
+                  1,
+                  1,
+                  "CFARepeatPatternDim" },
         };
 
         // Add missing dng fields
@@ -529,17 +589,88 @@ process_image_for_capture(const uint8_t *image, int count)
                 mp_get_device_make(),
                 mp_get_device_model());
         TIFFSetField(tif, TIFFTAG_UNIQUECAMERAMODEL, uniquecameramodel);
-        if (camera->colormatrix[0]) {
+
+        // Color matrices
+        if (camera->calibration.color_matrix_1[0]) {
+                TIFFSetField(tif,
+                             TIFFTAG_COLORMATRIX1,
+                             9,
+                             camera->calibration.color_matrix_1);
+        } else if (camera->colormatrix[0]) {
                 TIFFSetField(tif, TIFFTAG_COLORMATRIX1, 9, camera->colormatrix);
         } else {
                 TIFFSetField(tif, TIFFTAG_COLORMATRIX1, 9, colormatrix_srgb);
         }
-        if (camera->forwardmatrix[0]) {
-                TIFFSetField(tif, TIFFTAG_FORWARDMATRIX1, 9, camera->forwardmatrix);
+        if (camera->calibration.color_matrix_2[0]) {
+                TIFFSetField(tif,
+                             TIFFTAG_COLORMATRIX2,
+                             9,
+                             camera->calibration.color_matrix_2);
         }
+
+        if (camera->calibration.forward_matrix_1[0]) {
+                TIFFSetField(tif,
+                             DNGTAG_FORWARDMATRIX1,
+                             9,
+                             camera->calibration.forward_matrix_1);
+        } else if (camera->forwardmatrix[0]) {
+                TIFFSetField(tif, DNGTAG_FORWARDMATRIX1, 9, camera->forwardmatrix);
+        }
+
+        if (camera->calibration.forward_matrix_2[0]) {
+                TIFFSetField(tif,
+                             DNGTAG_FORWARDMATRIX2,
+                             9,
+                             camera->calibration.forward_matrix_2);
+        }
+
         static const float neutral[] = { 1.0, 1.0, 1.0 };
         TIFFSetField(tif, TIFFTAG_ASSHOTNEUTRAL, 3, neutral);
-        TIFFSetField(tif, TIFFTAG_CALIBRATIONILLUMINANT1, 21);
+        if (camera->calibration.illuminant_1) {
+                TIFFSetField(tif,
+                             TIFFTAG_CALIBRATIONILLUMINANT1,
+                             camera->calibration.illuminant_1);
+        } else {
+                TIFFSetField(tif, TIFFTAG_CALIBRATIONILLUMINANT1, 21);
+        }
+
+        if (camera->calibration.illuminant_2) {
+                TIFFSetField(tif,
+                             TIFFTAG_CALIBRATIONILLUMINANT2,
+                             camera->calibration.illuminant_2);
+        }
+
+        if (camera->calibration.tone_curve_length) {
+                TIFFSetField(tif,
+                             DCPTAG_PROFILE_TONE_CURVE,
+                             camera->calibration.tone_curve_length,
+                             camera->calibration.tone_curve);
+        }
+
+        if (camera->calibration.hue_sat_map_dims[0]) {
+                TIFFSetField(tif,
+                             DCPTAG_PROFILE_HUE_SAT_MAP_DIMS,
+                             3,
+                             camera->calibration.hue_sat_map_dims);
+                TIFFSetField(tif,
+                             DCPTAG_PROFILE_HUE_SAT_MAP_DATA_1,
+                             camera->calibration.hue_sat_map_dims[0] *
+                                     camera->calibration.hue_sat_map_dims[1] *
+                                     camera->calibration.hue_sat_map_dims[2] * 3,
+                             camera->calibration.hue_sat_map_data_1);
+                if (camera->calibration.hue_sat_map_data_2 != NULL) {
+                        TIFFSetField(
+                                tif,
+                                DCPTAG_PROFILE_HUE_SAT_MAP_DATA_2,
+                                camera->calibration.hue_sat_map_dims[0] *
+                                        camera->calibration.hue_sat_map_dims[1] *
+                                        camera->calibration.hue_sat_map_dims[2] * 3,
+                                camera->calibration.hue_sat_map_data_2);
+                }
+        }
+
+        TIFFSetField(tif, TIFFTAG_ANALOGBALANCE, 3, balance);
+
         // Write black thumbnail, only windows uses this
         {
                 unsigned char *buf =
@@ -561,18 +692,23 @@ process_image_for_capture(const uint8_t *image, int count)
         TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_CFA);
         TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, 1);
         TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
+
+#if (TIFFLIB_VERSION > 20230000)
         static const short cfapatterndim[] = { 2, 2 };
-        TIFFSetField(tif, TIFFTAG_CFAREPEATPATTERNDIM, cfapatterndim);
-#if (TIFFLIB_VERSION < 20201219) && !LIBTIFF_CFA_PATTERN
+        TIFFSetField(tif, TIFFTAG_CFAREPEATPATTERNDIM, 2, cfapatterndim);
         TIFFSetField(tif,
-                     TIFFTAG_CFAPATTERN,
+                     DNGTAG_CFAPATTERN,
+                     4,
                      mp_pixel_format_cfa_pattern(mode.pixel_format));
 #else
+        static const short cfapatterndim[] = { 2, 2 };
+        TIFFSetField(tif, TIFFTAG_CFAREPEATPATTERNDIM, cfapatterndim);
         TIFFSetField(tif,
-                     TIFFTAG_CFAPATTERN,
+                     DNGTAG_CFAPATTERN,
                      4,
                      mp_pixel_format_cfa_pattern(mode.pixel_format));
 #endif
+
         printf("TIFF version %d\n", TIFFLIB_VERSION);
         int whitelevel = camera->whitelevel;
         if (!whitelevel) {
@@ -932,6 +1068,8 @@ update_state(MPPipeline *pipeline, const struct mp_process_pipeline_state *state
         // gain_is_manual = state->gain_is_manual;
         gain = state->gain;
         gain_max = state->gain_max;
+        balance[0] = state->balance_red;
+        balance[2] = state->balance_blue;
 
         exposure_is_manual = state->exposure_is_manual;
         exposure = state->exposure;
